;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                            ;;
;; KERNEL32.INC                                               ;;
;;                                                            ;;
;; Included 32 bit kernel files for MenuetOS                  ;;
;;                                                            ;;
;; This file is kept separate as it will be easier to         ;;
;; maintain and compile with an automated SETUP program       ;;
;; in the future.                                             ;;
;;                                                            ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


; structure definition helper
macro struct name, [arg]
 {
  common
   name@struct fix name
   struc name arg {
 }

macro struct_helper name
 {
   virtual at 0
   name name
   sizeof.#name = $ - name
   name equ sizeof.#name
   end virtual
 }

ends fix } struct_helper name@struct


; Core functions

include "core/SYS32.INC"    ; process management
include "core/SCHED.INC"    ; process scheduling
include "core/FPU.INC"      ; FPU handler
include "core/SYSCALL.INC"  ; system call

; GUI stuff
include "gui/WINDOW.INC"
include "gui/EVENT.INC"
include "gui/FONT.INC"
include "gui/BUTTON.INC"

; shutdown

include "boot/SHUTDOWN.INC" ; shutdown or restart

; file system

include "fs/FS.INC"       ; syscall
include "fs/FAT32.INC"    ; read / write for fat32 filesystem
include "blkdev/RD.INC"       ; ramdisk read /write

; sound

include "sound/SB16.INC"     ; playback for Sound Blaster 16
include "sound/PLAYNOTE.INC" ; player Note for Speaker PC

; display

include "video/VESA12.INC"   ; Vesa 1.2 functions
include "video/VESA20.INC"   ; Vesa 2.0 functions
include "video/VGA.INC"      ; VGA 16 color functions

; Network Interface & TCPIP Stack

include "network/STACK.INC"

; Mouse pointer

include "gui/MOUSE.INC"

; Window skinning

include "gui/SKINCODE.INC"

; Pci functions

include "bus/pci/PCI32.INC"

; Floppy drive controller

include "blkdev/FDC.INC"
include "blkdev/CDROM.INC"

; Character devices

include "hid/KEYBOARD.INC"

;% -include
