;-----------------------------------------------------
;   PCI Device Enumeration
;
;   Compile with FASM for Menuet
;
;   This program will attempt to scan the PCI Bus
;   and display basic information about each device
;   connected to the PCI Bus.
;
;
;   Author:  Jason Delozier
;
;
;------------------------------------------------------

use32

               org    0x0

               db     'MENUET01'              ; 8 byte id
               dd     0x01                    ; header version
               dd     START                   ; start of code
               dd     I_END                   ; size of image
               dd     0x1000                  ; memory for app
               dd     0x1000                  ; esp
               dd     0x0 , 0x0               ; I_Param , I_Icon

START:                          ; start of execution
     call draw_window

still:
    mov  eax,10                 ; wait here for event
    int  0x40

    cmp  eax,1                  ; redraw request ?
    je   red
    cmp  eax,2                  ; key in buffer ?
    je   key
    cmp  eax,3                  ; button in buffer ?
    je   button

    jmp  still

  red:                          ; redraw
    call draw_window
    jmp  still
  key:                          ; key
    mov  eax,2                  ; just read it and ignore
    int  0x40
    jmp  still
  button:                       ; button
    mov  eax,17                 ; get id
    int  0x40

    cmp  ah,1                   ; button id=1 ?
    jne  noclose

    mov  eax,-1                 ; close this program
    int  0x40
  noclose:
    jmp  still




;   *********************************************
;   *******  WINDOW DEFINITIONS AND DRAW ********
;   *********************************************


draw_window:
    mov  eax,12                    ; function 12:tell os about windowdraw
    mov  ebx,1                     ; 1, start of draw
    int  0x40

                                   ; DRAW WINDOW
    mov  eax,0                     ; function 0 : define and draw window
    mov  ebx,100*65536+470         ; [x start] *65536 + [x size]
    mov  ecx,100*65536+420         ; [y start] *65536 + [y size]
    mov  edx,0x02ffffff            ; color of work area RRGGBB,8->color gl
    mov  esi,0x805080d0            ; color of grab bar  RRGGBB,8->color gl
    mov  edi,0x005080d0            ; color of frames    RRGGBB
    int  0x40

                                   ; WINDOW LABEL
    mov  eax,4                     ; function 4 : write text to window
    mov  ebx,8*65536+8             ; [x start] *65536 + [y start]
    mov  ecx,0x10ddeeff            ; font 1 & color ( 0xF0RRGGBB )
    mov  edx,labelt                ; pointer to text beginning
    mov  esi,labellen-labelt       ; text length
    int  0x40

                                   ; CLOSE BUTTON
    mov  eax,8                     ; function 8 : define and draw button
    mov  ebx,(470-19)*65536+12     ; [x start] *65536 + [x size]
    mov  ecx,5*65536+12            ; [y start] *65536 + [y size]
    mov  edx,1                     ; button id
    mov  esi,0x6688dd              ; button color RRGGBB
    int  0x40

    mov ebx, 20*65536+35           ;Draw work format
    mov ecx, 0x224466
    mov edx, PCIWin
    mov esi, 74
newline:
    mov eax, 4
    int 0x40
    add ebx, 10
    add edx, 74
    cmp [edx], byte 'x'
    jne newline


    call Get_PCI_Info
    mov cx, [PCI_Version]
    mov eax, 47
    mov ebx, 0x00040100
    mov esi, 0x00000000
    mov edx, 120*65536+55
    int 0x40

    mov cl, [PCI_LastBus]
    mov eax, 47
    mov ebx, 0x00020100
    mov esi, 0x00000000
    mov edx, 120*65536+65
    int 0x40

    call scan_Pci

    mov  eax,12                    ; function 12:tell os about windowdraw
    mov  ebx,2                     ; 2, end of draw
    int  0x40

    ret

Get_PCI_Info:
    mov eax, 62
    mov ebx, 0
    int 0x40
    mov [PCI_Version], ax

    mov eax, 62
    mov ebx, 1
    int 0x40
    mov [PCI_LastBus], al
ret



scan_Pci:
   cmp [PCI_LastBus],0xff
   jne Pci_Exists
   ret
Pci_Exists:
   mov [V_Bus], 0         ;reset varibles
   mov [V_Dev], 0         ;
   mov edx,  20*65536+110 ;start write position

Start_Enum:
   mov bl, 6         ;get Device ID / Vendor ID
   mov bh, [V_Bus]
   mov ch, [V_Dev]
   mov cl, 0
   mov eax, 62
   int 0x40

   cmp ax, 0         ;check next slot if nothing exists
   je nextDev
   cmp ax, 0xffff
   je nextDev

   mov [PCI_Vendor], ax
   shr eax, 16
   mov [PCI_Device], ax

   mov eax, 62            ;PCI Sys Function
   mov bl, 4              ;Read config byte
   mov bh, [V_Bus]        ;Bus #
   mov ch, [V_Dev]        ;Device # on bus
   mov cl, 0x08           ;Register to read (Get Revision)
   int 0x40               ;Read it
   mov [PCI_Rev], al      ;Save it

   mov eax, 62            ;PCI Sys Function
   mov cl, 0x0b           ;Register to read (Get class)
   int 0x40               ;Read it
   mov [PCI_Class], al    ;Save it

   mov eax, 62            ;PCI Sys Function
   mov cl, 0x0a           ;Register to read (Get Subclass)
   int 0x40               ;Read it
   mov [PCI_SubClass], al ;Save it

   call Print_New_Device

nextDev:
   inc [V_Dev]
   jno Start_Enum    ;jump if no overflow (<256)

   mov [V_Dev], 0
   inc [V_Bus]
   mov al, [PCI_LastBus]

   cmp byte [V_Bus], al
   jbe Start_Enum
ret

Print_New_Device:
   mov eax, 47    ;Write number to screen
   mov ebx, 0x00040100    ;4 byte number, print in hexidecimal
   mov esi, 0x00000000    ;Color of text
   mov cx, [PCI_Device]   ;Pointer to number to be written
   int 0x40               ;Write it

   push dx                ;Calculate position
   mov dx, 62             ;
   shl edx, 16            ;
   pop dx                 ;
   mov cx, [PCI_Vendor]   ;Write Vendor ID
   int 0x40               ;

   mov ebx, 0x00020100    ;2 byte number, in hexidecimal format
   push dx                ;move write cursor
   mov dx, 110            ;
   shl edx, 16            ;
   pop dx                 ;
   mov cl, [V_Bus]        ;write bus number
   int 0x40               ;

   push dx                ;move write cursor
   mov dx, 146            ;
   shl edx, 16            ;
   pop dx                 ;
   mov cl, [V_Dev]        ;Write device number
   int 0x40               ;

   push dx                ;move write cursor
   mov dx, 179            ;
   shl edx, 16            ;
   pop dx                 ;
   mov cl, [PCI_Rev]      ;Draw Revision to screen
   int 0x40               ;

   push dx                ;move write cursor
   mov dx, 215            ;
   shl edx, 16            ;
   pop dx                 ;
   mov cl, [PCI_Class]    ;Draw Class to screen
   int 0x40               ;

   push dx                ;move write curson
   mov dx, 266            ;
   shl edx, 16            ;
   pop dx                 ;
   mov cl, [PCI_SubClass] ;Draw Sub Class to screen
   int 0x40               ;

                          ;Write Descriptions
   push dx                ;remember our current write line
   mov esi, 26            ;length of text
   mov bx, 310            ;X-position of text on screen
   shl ebx, 16            ;
   mov bx, dx             ;Current Y position
   mov ecx, 0             ;color of text

   xor eax, eax           ;calculate pointer to string
   mov al, [PCI_Class]    ;Determine Description based on class
   mov dl,  26            ;Each description is 26 characters long
   mul dl                 ;get offset to correct description
   mov edx, Descriptions  ;starting point of description text
   add edx, eax           ;calculated corresponding description
   mov eax, 4             ;draw text sys function
   int 0x40               ;draw it

   mov dx, 20             ;Starting Y coordinate
   shl edx, 16            ;
   pop dx                 ;
   add edx, 10            ;Next line down
ret



; DATA AREA
V_Bus        db 0
V_Dev        db 0
PCI_Version  dw 0
PCI_LastBus  db 0
PCI_Device   dw 0
PCI_Vendor   dw 0
PCI_Bus      db 0
PCI_Dev      db 0
PCI_Rev      db 0
PCI_Class    db 0
PCI_SubClass db 0

labelt:
       db 'PCI Device Enumeration'
labellen:

PCIWin:
db 'Please remember to enable PCI Access to Applications in Setup Menu.       '
db '                                                                          '
db 'PCI Version  =                                                            '
db 'Last PCI Bus =                                                            '
db '                                                                          '
db 'DevID  VenID  Bus#  Dev#  Rev  Class  Subclass  Description               '
db '-----  -----  ----  ----  ---  -----  --------  --------------------------'
db 'x'

Descriptions:
db 'Unknown                   '
db 'Storage                   '
db 'Network                   '
db 'Display                   '
db 'Multimedia                '
db 'Memory                    '
db 'Bridge                    '
db 'Simple Communication      '
db 'System                    '
db 'Input                     '
db 'Docking                   '
db 'Processor                 '
db 'Serial                    '
db 'Wireless                  '
db 'Intelligent I/O           '
db 'Satellite Communication   '
db 'En/Decryption             '
db 'Data Aqu & Signal Process '

I_END:







